try:
    from django.forms.utils import ErrorDict
except ImportError:
    from django.forms.util import ErrorDict  # type: ignore[no-redef, import-untyped] # noqa: E501

from django.utils.html import format_html

from .angular_base import NgFormBaseMixin, SafeTuple


class NgModelFormMixin(NgFormBaseMixin):
    """
    Add this NgModelFormMixin to every class derived from ``forms.Form``, if that custom ``Form``
    shall be managed through an Angular controller.
    It adds attributes ``ng-model``, and optionally ``ng-change``, ``ng-class`` and ``ng-style``
    to each of your input fields.
    If form validation fails, the ErrorDict is rewritten in a way, so that the Angular controller
    can access the error strings using the same key values as for its models.
    """  # noqa: E501

    add_djng_error = False

    def __init__(self, data=None, *args, **kwargs):
        self.scope_prefix = kwargs.pop(
            "scope_prefix",
            getattr(self, "scope_prefix", None),
        )
        self.ng_directives = {}
        for key in list(kwargs.keys()):
            if key.startswith("ng_"):
                fmtstr = kwargs.pop(key)
                self.ng_directives[key.replace("_", "-")] = fmtstr
        if hasattr(self, "Meta") and hasattr(self.Meta, "ng_models"):
            if not isinstance(getattr(self.Meta, "ng_models"), list):
                raise TypeError("Meta.ng_model is not of type list")
        elif "ng-model" not in self.ng_directives:
            self.ng_directives["ng-model"] = "%(model)s"
        self.prefix = kwargs.get("prefix")
        if self.prefix and data:
            if data.get(self.prefix):
                data = {
                    self.add_prefix(name): value
                    for (name, value) in data.get(self.prefix).items()
                }
            else:
                data = {
                    name: value
                    for (name, value) in data.items()
                    if name.startswith(self.prefix + ".")
                }
        super().__init__(data, *args, **kwargs)
        if self.scope_prefix == self.form_name:
            raise ValueError(
                "The form's name may not be identical with its scope_prefix",
            )

    def _post_clean(self):
        """
        Rewrite the error dictionary, so that its keys correspond to
        the model fields.
        """
        super()._post_clean()
        if self._errors and self.prefix:
            self._errors = ErrorDict(
                (self.add_prefix(name), value)
                for name, value in self._errors.items()
            )

    def get_initial_data(self):
        """
        Return a dictionary specifying the defaults for this form. This
        dictionary can be used to inject the initial values for an Angular
        controller using the directive:
        ``ng-init={{ thisform.get_initial_data|js|safe }}``.
        """
        data = {}
        ng_models = (
            hasattr(self, "Meta") and getattr(self.Meta, "ng_models", []) or []
        )
        for name, field in self.fields.items():
            if "ng-model" in self.ng_directives or name in ng_models:
                data[name] = (
                    self.initial.get(name) if self.initial else field.initial
                )
        return data

    def get_field_errors(self, field):
        errors = super().get_field_errors(field)
        if field.is_hidden:
            return errors
        identifier = format_html("{0}.{1}", self.form_name, field.html_name)
        errors.append(
            SafeTuple(
                (
                    identifier,
                    self.field_error_css_classes,
                    "$pristine",
                    "$message",
                    "invalid",
                    "$message",
                ),
            ),
        )
        return errors

    def non_field_errors(self):
        errors = super().non_field_errors()
        errors.append(
            SafeTuple(
                (
                    self.form_name,
                    self.form_error_css_classes,
                    "$pristine",
                    "$message",
                    "invalid",
                    "$message",
                ),
            ),
        )
        return errors

    def get_widget_attrs(self, bound_field):
        attrs = super().get_widget_attrs(bound_field)
        identifier = self.add_prefix(bound_field.name)
        ng = {
            "name": bound_field.name,
            "identifier": identifier,
            "model": (
                ("%s['%s']" % (self.scope_prefix, identifier))
                if self.scope_prefix
                else identifier
            ),
        }
        if hasattr(self, "Meta") and bound_field.name in getattr(
            self.Meta,
            "ng_models",
            [],
        ):
            attrs["ng-model"] = ng["model"]
        for key, fmtstr in self.ng_directives.items():
            attrs[key] = fmtstr % ng
        return attrs



# Generated by Django 1.10.5 on 2017-01-23 11:41

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0009_delete_corporateimage"),
    ]



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import hashlib
import operator
from functools import reduce

from django.conf import settings
from django.contrib.auth.models import Group, Permission, User
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.core.validators import validate_email
from django.db import models
from django.db.models import Q
from django.utils.encoding import smart_str
from django.utils.translation import gettext_lazy as _

from codenerix.helpers import clean_memcache_item
from codenerix.middleware import get_current_user
from codenerix.models import CodenerixModel, GenLog


class GenPerson(GenLog, models.Model):  # META: Abstract class
    """
    Defines a person
    """

    # Control fields
    user = models.OneToOneField(  # type: ignore[var-annotated]
        User,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="person",
    )
    name = models.CharField(  # type: ignore[var-annotated]
        _("Name"),
        max_length=45,
        blank=False,
        null=False,
    )
    surname = models.CharField(  # type: ignore[var-annotated]
        _("Surname"),
        max_length=90,
        blank=False,
        null=False,
    )
    disabled = models.DateTimeField(  # type: ignore[var-annotated]
        _("Disabled from"),
        null=True,
        blank=True,
    )
    creator = models.ForeignKey(  # type: ignore[var-annotated]
        User,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="creators",
        default=None,
    )

    class Meta(CodenerixModel.Meta):
        abstract = True

    def __str__(self):
        if self.name and self.surname:
            output = "%s %s" % (smart_str(self.name), smart_str(self.surname))
        elif self.name:
            output = self.name
        elif self.surname:
            output = self.surname
        else:
            output = "%s*" % (self.user)
        return smart_str(output)

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(("user__username", _("User")))
        fields.append(("name", _("Name"), 100))
        fields.append(("surname", _("Surname"), 100))
        fields.append(("user__email", _("Email"), 100))
        fields.append(("user__date_joined", _("Created"), 80))
        fields.append(("user__is_active", _("Active"), None, "center"))
        fields.append(("disabled", _("Disabled from")))
        return fields

    def __limitQ__(self, info):  # noqa: N802
        limit = {}
        # If user is not a superuser, the shown records depends on the profile
        if not info.request.user.is_superuser:
            # All the people records shown below must be related with this user
            criterials = []

            # The criterials are not exclusives
            if criterials:
                limit["profile_people_limit"] = reduce(
                    operator.or_,
                    criterials,
                )

        return limit

    def __searchF__(self, info):  # noqa: N802
        tf = {}
        tf["user__is_active"] = (
            _("Active"),
            lambda x: Q(user__is_active=x),
            [(True, _("Yes")), (False, _("No"))],
        )
        return tf

    def __searchQ__(self, info, text):  # noqa: N802
        tf = {}
        tf["username"] = Q(user__username__icontains=text.lower())
        tf["name"] = Q(name__icontains=text)
        tf["surname"] = Q(surname__icontains=text)
        tf["user_email"] = Q(user__email__icontains=text)
        tf["user__date_joined"] = "datetime"
        return tf

    def is_admin(self):
        try:
            return bool(
                self.user.is_superuser or self.user.groups.get(name="Admins"),
            )
        except Exception:
            return False

    def profiles(self):
        """
        return the rolls this people is related with
        """
        limit = []

        if self.is_admin():
            limit.append(_("Administrator"))
        limit.sort()

        return limit

    def delete(self):
        self.clean_memcache()
        return super().delete()

    def lock_delete(self, request=None):
        if request is not None:
            if request.user == self.user:
                return _("Cannot delete to yourself")
            else:
                return super().lock_delete()
        else:
            return super().lock_delete()

    def clean_memcache(self):
        if self.pk:
            prefix = hashlib.md5(usedforsecurity=False)
            prefix.update(
                base64.b64encode(settings.SECRET_KEY.encode("utf-8")),
            )
            clean_memcache_item(
                "person:{}".format(self.pk),
                prefix.hexdigest(),
            )

    def get_grouppermit(self):
        return self.user.groups.all()

    def presave(self, username, password, email, confirm, unusable=False):
        self.clean_memcache()
        # Check username
        # Rewrite username
        username = username.lower()

        # if exist self.user no modify username
        if not self.user and not username:
            raise ValidationError(
                "This person doesn't have a user assigned, can "
                "not change password or email without an user",
            )

        # Check passwords
        if not unusable:
            if password and confirm and password != confirm:
                raise ValidationError("Passwords do not match")
            if password and (len(password) < settings.PASSWORD_MIN_SIZE):
                raise ValidationError(
                    "Password can not be smaller than {} characters".format(
                        settings.PASSWORD_MIN_SIZE,
                    ),
                )

        # Check email
        if email:
            validate_email(email)
        else:
            raise ValidationError(
                "Missing email when saving a new person without a "
                "preassigned user",
            )

        # Create user and save it to the database
        if self.user:
            # Check if the user requested to change the username
            if username != self.user.username:
                # Check if the username already exists in the database and
                # it is not me
                already = (
                    User.objects.filter(username=username)
                    .exclude(pk=self.user.pk)
                    .first()
                )
                if already:
                    # Username already in use
                    raise ValidationError(
                        "Username already exists in the database",
                    )
                else:
                    # Set new username
                    self.user.username = username

            # Update password if any
            if unusable:
                self.user.set_unusable_password()

            elif password:
                self.user.set_password(password)
            # Update email address
            self.user.email = email
        else:
            # Extra args
            if unusable:
                extra_args = {}
            else:
                extra_args = {"password": password}
            # Create new user
            self.user = User.objects.create_user(
                username=username,
                email=email,
                **extra_args,
            )
            if unusable:
                self.user.set_unusable_password()
            # user creator of the user
            self.creator = get_current_user()

        # Save changes
        self.user.save()

    def refresh_permissions(self, quiet=False):
        # Check we have a user to work with
        if self.user:
            # Clear groups and permisions for this user
            self.user.groups.clear()
            self.user.user_permissions.clear()

            # Collect all groups and unique permissions for this
            # user relationships
            groups = []
            permissions = []
            for x in self._meta.get_fields():
                model = x.related_model

                # Only check roles
                if model and issubclass(model, GenRole):
                    # Get the link
                    link = getattr(self, x.name, None)

                    # Check if the linked class has CodenerixMeta
                    if link and hasattr(link, "CodenerixMeta"):
                        # Get groups and permissions from that class
                        groups += list(
                            getattr(link.CodenerixMeta, "rol_groups", None)
                            or {},
                        )
                        permissions += list(
                            getattr(
                                link.CodenerixMeta,
                                "rol_permissions",
                                None,
                            )
                            or [],
                        )

            # Add groups
            for groupname in set(groups):
                group = Group.objects.filter(name=groupname).first()
                if group is None:
                    # Group not found, remake permissions for all groups with
                    # roles
                    GenPerson.group_permissions(type(self))
                    # Check again
                    group = Group.objects.filter(name=groupname).first()
                    if group is None:
                        raise OSError(
                            f"Group '{groupname} not found in the system. I "
                            "have tried to remake groups but this group is "
                            "not defined as a Role and it doesn't belong to "
                            "the system either",
                        )

                # Add the user to this group
                self.user.groups.add(group)

            # Add permissions
            for permissionname in set(permissions):
                permission = Permission.objects.filter(
                    codename=permissionname,
                ).first()
                if permission is None:
                    raise OSError(
                        "Permission '{}' not found in the system".format(
                            permissionname,
                        ),
                    )

                # Add the permission to this user
                self.user.user_permissions.add(permission)

        else:
            if not quiet:
                raise OSError(
                    "You can not refresh permissions for a Person wich "
                    "doesn't have an associated user",
                )

    @staticmethod
    def group_permissions(clss):
        groupsresult = {}
        for x in clss._meta.get_fields():
            model = x.related_model

            # Check if it is a role
            if model and issubclass(model, GenRole):
                # Check if the class has CodenerixMeta
                if hasattr(model, "CodenerixMeta"):
                    # Get groups and permissions
                    groups = getattr(model.CodenerixMeta, "rol_groups", [])

                    # Add groups
                    if groups:
                        groups_is_dict = type(groups) is dict
                        groupslist = list(set(groups))
                        for groupname in groupslist:
                            # Check permissions just in case something is wrong
                            perms = []
                            if groups_is_dict:
                                for permname in groups[groupname]:
                                    perm = Permission.objects.filter(
                                        codename=permname,
                                    ).first()
                                    # Commented by Juan Soler on a project we
                                    # both are working on. Since this code is
                                    # preventing a proper execution, I will
                                    # comment it as he has done in our project
                                    # and I will wait for him to validate this
                                    # lines
                                    # if perm is None:
                                    #    raise IOError("Permission '{}' not found for group '{}'!".format(permname, groupname)) # noqa: E501
                                    # else:
                                    if perm is not None:
                                        perms.append(perm)

                            # Remember perms for this group
                            if groupname not in groupsresult:
                                groupsresult[groupname] = []
                            groupsresult[groupname] += perms

        # Set permissions for all groups
        for groupname in groupsresult:
            # Get group
            group = Group.objects.filter(name=groupname).first()
            if group is None:
                # Add group if doesn't exists
                group = Group(name=groupname)
                group.save()
            else:
                # Remove all permissions for this group
                group.permissions.clear()

            # Add permissions to the group
            for perm in groupsresult[groupname]:
                group.permissions.add(perm)

    def save(self, *args, **kwargs):
        # Save this person
        answer = super().save(*args, **kwargs)
        # Refresh permissions if possible
        self.refresh_permissions(quiet=True)
        # Return answer
        return answer


class GenRole:
    def save(self, *args, **kwargs):
        # only update permissions for new users
        refresh_permissions = not self.pk
        result = super().save(*args, **kwargs)
        if refresh_permissions:
            person = self.__CDNX_search_person_CDNX__()
            if person:
                person.refresh_permissions(quiet=True)
        return result

    def delete(self, using=None, keep_parents=False):
        person = self.__CDNX_search_person_CDNX__()
        result = super().delete(using=using, keep_parents=keep_parents)
        if person:
            # update permissions
            person.refresh_permissions(quiet=True)
        return result

    def __CDNX_search_person_CDNX__(self):  # noqa: N802
        # search relation with GenPerson
        person = None
        for field in self._meta.get_fields():
            model = field.related_model
            if model and issubclass(model, GenPerson):
                try:
                    person = getattr(self, field.name)
                except ObjectDoesNotExist:
                    pass
                break
        return person

    def CDNX_refresh_permissions_CDNX(self):  # noqa: N802
        person = self.__CDNX_search_person_CDNX__()
        if person:
            person.refresh_permissions(quiet=True)



# Generated by Django 1.10.6 on 2017-04-28 08:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0016_auto_20170419_1533"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_flag",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "Add"), (2, "Change"), (3, "Delete")],
                verbose_name="Action",
            ),
        ),
    ]



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

try:
    from subprocess import getstatusoutput

    pythoncmd = "python3"
except Exception:
    from commands import (  # type: ignore[import-not-found,no-redef]
        getstatusoutput,
    )

    pythoncmd = "python"

from codenerix_lib.debugger import Debugger
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand, Debugger):
    # Show this when the user types help
    help = "Used to build all locales details"

    def add_arguments(self, parser):
        # Set witch mode to use
        parser.add_argument(
            "--mode",
            dest="mode",
            default="suexec",
            help="Mode used in the environment 'suexec', 'apache' "
            "or 'wwwdata'",
        )

        # Named (optional) arguments
        parser.add_argument(
            "--noauto",
            action="store_true",
            dest="noauto",
            default=False,
            help="Tells the command not to find automatic solution "
            "for problems",
        )

        # Named (optional) arguments
        parser.add_argument(
            "--noguess",
            action="store_true",
            dest="noguess",
            default=False,
            help="Disable guessing user environment",
        )

        # Named (optional) arguments
        parser.add_argument(
            "--clean",
            action="store_true",
            dest="clean",
            default=False,
            help="Do a full clean by deleting everything before "
            "building translations",
        )

        # Named (optional) arguments
        parser.add_argument(
            "--compile",
            action="store_true",
            dest="compile",
            default=False,
            help="Compile .po files",
        )

    def handle(self, *args, **options):
        # Check arguments
        mode = options["mode"]
        if mode not in ["wwwdata", "apache", "suexec"]:
            self.print_help("", "")
            raise CommandError(
                "Allowed modes are suexec, apache or wwwdata ('apache' and "
                "'wwwdata' option will force permissions to apache or "
                "www-data user, suexec won't touch permissions)",
            )

        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()

        # Get environment
        appname = settings.ROOT_URLCONF.split(".")[0]
        basedir = settings.BASE_DIR
        appdir = os.path.abspath("{}/{}".format(basedir, appname))
        noauto = options["noauto"]

        # Check user selection
        if not options["noguess"]:
            cmd = r"find {} -name 'locale' -exec ls -lR {{}} \; | grep www-data".format(  # noqa: E501
                appdir,
            )
            status, output = getstatusoutput(cmd)
            if status:
                guess = "suexec"
            else:
                guess = "wwwdata"
            if guess != mode:
                self.print_help("", "")
                raise CommandError(
                    f"You have selected mode '{mode}' but I believe you are "
                    "using '{guess}', if sure use --noguess",
                )

        # Show header
        self.debug("Creating locales for {}".format(appname), color="blue")
        if noauto:
            self.debug("Autoconfig mode is OFF", color="yellow")

        # Get list of apps
        apps = []
        length = len(appname)
        for app in settings.INSTALLED_APPS:
            if app[0:length] == appname:
                apps.append(app[length + 1 :])

        # Check for locale folders
        error = False
        for app in [""] + apps:
            testpath = os.path.abspath(
                "{}/{}/locale".format(appdir, app).replace("//", "/"),
            )
            if not os.path.exists(testpath):
                if noauto:
                    error = True
                    self.debug(
                        "'locale' folder missing at {}/".format(testpath),
                        color="yellow",
                    )
                else:
                    self.debug(
                        "'locale' folder missing, creating {}/".format(
                            testpath,
                        ),
                        color="purple",
                    )
                    os.mkdir(testpath)

        if error:
            raise CommandError(
                "Some error has happened, can not keep going! (avoid "
                "using --noauto to let CODENERIX find a solution)",
            )

        # Check execution mode
        sudo = ""
        if mode == "apache" or mode == "wwwdata":
            status, output = getstatusoutput("whoami")
            if status:
                # Error in command
                self.error("Error while executing 'whoami' command")
                raise CommandError(output)
            elif output == "www-data":
                # Detected we are already www-data user, so keep going
                self.debug("Detected we are 'www-data' user", color="purple")
            else:
                # No permissions try to become root using sudo
                status, output = getstatusoutput("sudo whoami")
                if status:
                    # Error in command
                    self.error("Error while executing 'sudo whoami' command")
                    raise CommandError(output)
                elif output == "root":
                    # Detected sudo is working and we can become root
                    self.debug(
                        "Detected we can become 'root' user",
                        color="purple",
                    )
                    sudo = "sudo "
                else:
                    # No permissions
                    raise CommandError(
                        "You requested 'apache' or 'wwwdata' execution mode "
                        "but you are not www-data and we can not become root",
                    )

        # Delete locale folders if requested
        if options["clean"]:
            # Ask the user if would really like to remove all locale folders
            key = ""
            while key not in ["n", "y"]:
                self.debug(
                    "All 'locale' folders are going to be removed, "
                    "are you sure? (y|n) ",
                    tail=False,
                    color="red",
                )
                try:
                    key = raw_input().lower()
                except NameError:
                    key = input().lower()
                self.debug("", header=False)

            # Remove all locale folders
            if key == "y":
                self.debug("Removing locale folders...", color="red")
                for app in [""] + apps:
                    testpath = os.path.abspath(
                        "{}/{}/locale".format(appdir, app).replace("//", "/"),
                    )
                    if os.path.exists(testpath):
                        self.debug(
                            "    > Removing {}".format(testpath),
                            color="red",
                        )
                        cmd = "{}rm -R {}/*".format(sudo, testpath)
                        status, output = getstatusoutput(cmd)
                        if status:
                            raise CommandError(output)
            else:
                raise CommandError(
                    "You requested to clean all 'locale' folders but "
                    "you answered NO to the previous question, can not "
                    "keep going!",
                )

        # Ready to go
        if not options["compile"]:
            for code, name in settings.LANGUAGES:
                self.debug(
                    "Processing translations for {}...".format(name),
                    color="cyan",
                )
                cmd = (
                    f"{sudo}{basedir}/manage.py makemessages -v0 "
                    f"--symlinks --ignore env -l {code}",
                )
                status, output = getstatusoutput(cmd)
                if status:
                    raise CommandError(output)

                cmd = (
                    f"{sudo}{basedir}/manage.py makemessages -v0 "
                    f"--symlinks --ignore env -d djangojs -l {code}"
                )
                status, output = getstatusoutput(cmd)
                if status:
                    raise CommandError(output)

        # Set permissions for locale folders
        if sudo:
            self.debug("Setting permissions...", color="cyan")
            if mode == "apache":
                user = "apache"
            elif mode == "wwwdata":
                user = "www-data"
            else:
                raise CommandError("Wrong mode for sudo '{}'".format(mode))
            for app in [""] + apps:
                testpath = os.path.abspath(
                    "{}/{}/locale".format(appdir, app).replace("//", "/"),
                )
                cmd = "sudo chown {}.{} {}/ -R".format(user, user, testpath)
                status, output = getstatusoutput(cmd)
                if status:
                    raise CommandError(output)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0004_auto_20160824_0757"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_flag",
            field=models.PositiveSmallIntegerField(verbose_name="Acci\xf3n"),
        ),
    ]



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from uuid import uuid4

from django.conf import settings
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from codenerix import __version__


def codenerix(request):
    """
    Codenerix CONTEXT
    """
    # Get values
    DEBUG = getattr(settings, "DEBUG", False)  # noqa: N806
    VERSION = getattr(  # noqa: N806
        settings,
        "VERSION",
        _(
            "WARNING: No version set to this code, "
            "add VERSION contant to your configuration",
        ),
    )

    # Set environment
    return {
        "DEBUG": DEBUG,
        "VERSION": VERSION,
        "CODENERIX_VERSION": __version__,
        "CODENERIX_UUID": str(uuid4()),
    }


def codenerix_js(request):
    cnf = {}

    # Get values
    CONNECTION_ERROR = getattr(  # noqa: N806
        settings,
        "CONNECTION_ERROR",
        None,
    )
    USER_BEHAVIOUR = getattr(settings, "USER_BEHAVIOUR", None)  # noqa: N806
    ALARMS_LOOPTIME = getattr(settings, "ALARMS_LOOPTIME", None)  # noqa: N806
    ALARMS_QUICKLOOP = getattr(  # noqa: N806
        settings,
        "ALARMS_QUICKLOOP",
        None,
    )
    ALARMS_ERRORLOOP = getattr(  # noqa: N806
        settings,
        "ALARMS_ERRORLOOP",
        None,
    )
    DEBUG = getattr(settings, "DEBUG", False)  # noqa: N806
    DATERANGEPICKER_OPTIONS = getattr(  # noqa: N806
        settings,
        "DATERANGEPICKER_OPTIONS",
        None,
    )
    DATETIME_RANGE_FORMAT = getattr(  # noqa: N806
        settings,
        "DATETIME_RANGE_FORMAT",
        None,
    )

    # Set values
    if USER_BEHAVIOUR is not None:
        cnf["user_behaviour"] = USER_BEHAVIOUR is True
    if CONNECTION_ERROR is not None:
        cnf["connection_error"] = CONNECTION_ERROR
    if ALARMS_LOOPTIME is not None:
        cnf["alarms_looptime"] = ALARMS_LOOPTIME
    if ALARMS_QUICKLOOP is not None:
        cnf["alarms_quickloop"] = ALARMS_QUICKLOOP
    if ALARMS_ERRORLOOP is not None:
        cnf["alarms_errorloop"] = ALARMS_ERRORLOOP
    cnf["debug"] = str(DEBUG).lower()
    cnf["session_cookie_age"] = getattr(settings, "SESSION_COOKIE_AGE", None)
    cnf["codenerix_css"] = getattr(
        settings,
        "CODENERIX_CSS",
        _("WARNING: CODENERIX_CSS is not set in your configuration!!!"),
    )
    cnf["codenerix_js"] = getattr(
        settings,
        "CODENERIX_JS",
        _("WARNING: CODENERIX_JS is not set in your configuration!!!"),
    )

    # Set daterange
    if (DATERANGEPICKER_OPTIONS is None) or (DATETIME_RANGE_FORMAT is None):
        daterangepicker = '"'
        if DATERANGEPICKER_OPTIONS is None:
            daterangepicker += "{}".format(
                _(
                    " WARNING: DATERANGEPICKER_OPTIONS is not "
                    "set in your configuration!!! ",
                ),
            )
        if DATETIME_RANGE_FORMAT is None:
            daterangepicker += "{}".format(
                _(
                    " WARNING: DATETIME_RANGE_FORMAT is not "
                    "set in your configuration!!! ",
                ),
            )
        daterangepicker += '"'
        cnf["daterangepicker"] = mark_safe(daterangepicker)
    else:
        cnf["daterangepicker"] = mark_safe(
            settings.DATERANGEPICKER_OPTIONS.format(
                Format=settings.DATETIME_RANGE_FORMAT[1],
                From=_("From"),
                To=_("To"),
                Apply=_("Apply"),
                Cancel=_("Cancel"),
                Su=_("Su"),
                Mo=_("Mo"),
                Tu=_("Tu"),
                We=_("We"),
                Th=_("Th"),
                Fr=_("Fr"),
                Sa=_("Sa"),
                January=_("January"),
                February=_("February"),
                March=_("March"),
                April=_("April"),
                May=_("May"),
                June=_("June"),
                July=_("July"),
                August=_("August"),
                September=_("September"),
                October=_("October"),
                November=_("November"),
                December=_("December"),
            ),
        )
    # Return environment
    return {"cnf": cnf}



#!/usr/bin/env python3
#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Deprecated library in favor for codenerix-lib
"""

import inspect
import sys
from os import getcwd

from codenerix_lib.colors import *  # noqa: F403,F401

# Show a warning message with information about who is importing this library
cf = inspect.currentframe()
me = True
who = None
while cf:
    if cf.f_code.co_name == "<module>":
        if me:
            me = False
        else:
            who = (
                cf.f_code.co_filename.replace(getcwd(), "."),
                cf.f_lineno,
                cf.f_code.co_name,
            )
            break
    cf = cf.f_back
if not who:
    who = ("<unknown>", 0, "<unknown>")

print(
    f"""

========================== W A R N I N G =============================
DJANGO-CODENERIX: you have imported 'lib.colors' which is DEPRECATED
it will not work in newer versions, instead use 'codenerix_lib.colors'
from codenerix-lib package", please update your code. This message was
shown because is being imported by:
'{who[0]}' at line {who[1]}
======================================================================

      """,
    file=sys.stderr,
)



# Generated by Django 1.10.6 on 2017-04-19 15:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0015_auto_20170418_1515"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_flag",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "A\xf1adir"), (2, "Cambiar"), (3, "Borrar")],
                verbose_name="Acci\xf3n",
            ),
        ),
    ]



# Generated by Django 1.10.5 on 2017-01-23 10:14
# Simplified by Juanmi Taboada on 2017-14-02 11:12

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0008_auto_20170119_1526"),
    ]

    operations = []



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Log",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "action_time",
                    models.DateTimeField(auto_now=True, verbose_name="Date"),
                ),
                (
                    "object_id",
                    models.TextField(
                        null=True,
                        verbose_name="Object id",
                        blank=True,
                    ),
                ),
                (
                    "object_repr",
                    models.CharField(
                        max_length=200,
                        verbose_name="Object repr",
                    ),
                ),
                (
                    "action_flag",
                    models.PositiveSmallIntegerField(verbose_name="Acci\xf3n"),
                ),
                (
                    "change_json",
                    models.TextField(verbose_name="Json", blank=True),
                ),
                (
                    "change_txt",
                    models.TextField(verbose_name="Txt", blank=True),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=models.DO_NOTHING,
                        blank=True,
                        to="contenttypes.ContentType",
                        null=True,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=models.DO_NOTHING,
                        blank=True,
                        to=settings.AUTH_USER_MODEL,
                        null=True,
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("list_log", "Can list log"),
                    ("detail_log", "Can view log"),
                ),
            },
        ),
    ]



# Generated by Django 1.10.6 on 2017-07-13 15:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0017_auto_20170428_0850"),
    ]

    operations = [
        migrations.AddField(
            model_name="log",
            name="snapshot_txt",
            field=models.TextField(
                blank=True,
                verbose_name=b"Snapshot Txt",  # type: ignore[arg-type]
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0002_auto_20160808_0843"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_time",
            field=models.DateTimeField(auto_now=True, verbose_name="Date"),
        ),
    ]



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
from datetime import datetime

from captcha import client  # type: ignore[import-not-found]
from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.forms.widgets import Textarea
from django.utils.encoding import smart_str
from django.utils.translation import gettext as _

from codenerix.multi_email_field.forms import (
    MultiEmailField as MultiEmailFormField,
)
from codenerix.widgets import (
    BootstrapWysiwygInput,
    Date2TimeInput,
    FileAngularInput,
    GenReCaptchaInput,
    MultiBlockWysiwygInput,
    WysiwygAngularInput,
)


class FileAngularField(models.FileField):
    description = "File manage throught the Angular service system"

    def formfield(self, **kwargs):
        defaults = {"widget": FileAngularInput}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class ImageAngularField(models.ImageField):
    description = "Image field for Angular JS"

    def formfield(self, **kwargs):
        defaults = {"widget": FileAngularInput}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class Date2TimeField(models.DateTimeField):
    def formfield(self, **kwargs):
        defaults = {"widget": Date2TimeInput}
        defaults.update(kwargs)
        return super().formfield(**defaults)

    def clean(self, *args, **kwargs):
        date_min = datetime(2000, 1, 1, 0, 0, 0)

        if args[0]:
            if type(args[0]) is not datetime:
                raise ValidationError(_("Invalid date"))
            elif args[0] and date_min.date() > args[0].date():
                raise ValidationError(_("Date too old"))
            # return args[0]
        return super().clean(*args, **kwargs)


email_er = r"[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*"  # noqa: E501


class MultiEmailField(MultiEmailFormField):
    message = _("Enter valid email addresses.")  # type: ignore[assignment]
    widget = Textarea(  # type: ignore[assignment]
        attrs={"ng-pattern": "/^({0})(\\n{0})*$/".format(email_er)},
    )

    def to_python(self, value):
        if value:
            for char in [",", " ", "\r"]:
                value = value.replace(char, "\n").replace("\n\n", "\n")
        return super().to_python(value)


class WysiwygAngularField(models.TextField):
    description = "A hand of cards (bridge style)"

    def formfield(self, **kwargs):
        defaults = {"widget": WysiwygAngularInput}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class MultiBlockWysiwygField(models.TextField):
    description = "Multi block WYSIWYG"

    def formfield(self, **kwargs):
        defaults = {"widget": MultiBlockWysiwygInput}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class BootstrapWysiwygField(models.TextField):
    description = "Bootstrap WYSIWYG"

    def formfield(self, **kwargs):
        defaults = {"widget": BootstrapWysiwygInput}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class GenReCaptchaField(forms.CharField):
    description = "ReCaptcha field throught the Angular service system"
    default_error_messages = {
        "captcha_invalid": _("Incorrect, please try again."),
        "captcha_error": _("Error verifying input, please try again."),
    }

    def __init__(
        self,
        fieldname=None,
        public_key=None,
        private_key=None,
        use_ssl=None,
        attrs={},
        *args,
        **kwargs,
    ):
        """
        ReCaptchaField can accepts attributes which is a dictionary of
        attributes to be passed to the ReCaptcha widget class. The widget will
        loop over any options added and create the RecaptchaOptions
        JavaScript variables as specified in
        https://code.google.com/apis/recaptcha/docs/customization.html

        On compiled verstion of the library, don't forget to add before the codenerix.js scripts to add:
        <script type="text/javascript" src="https://www.google.com/recaptcha/api.js?onload=vcRecaptchaApiLoaded&render=explicit" async defer></script>$
        """  # noqa: E501
        public_key = (
            public_key if public_key else settings.RECAPTCHA_PUBLIC_KEY
        )
        self.private_key = (
            private_key if private_key else settings.RECAPTCHA_PRIVATE_KEY
        )
        self.use_ssl = (
            use_ssl
            if use_ssl is not None
            else getattr(settings, "RECAPTCHA_USE_SSL", False)
        )

        self.widget = GenReCaptchaInput(
            fieldname=fieldname,
            public_key=public_key,
            use_ssl=self.use_ssl,
            attrs=attrs,
        )
        self.required = True
        return super().__init__(*args, **kwargs)

    def get_remote_ip(self):
        f = sys._getframe()
        while f:
            if "request" in f.f_locals:
                request = f.f_locals["request"]
                if request:
                    remote_ip = request.META.get("REMOTE_ADDR", "")
                    forwarded_ip = request.META.get("HTTP_X_FORWARDED_FOR", "")
                    ip = remote_ip if not forwarded_ip else forwarded_ip
                    return ip
            f = f.f_back

    def clean(self, values):
        super().clean(values[0])
        recaptcha_challenge_value = smart_str(values[0])
        recaptcha_response_value = smart_str(values[1])

        if (
            os.environ.get("RECAPTCHA_TESTING", None) == "True"
            and recaptcha_response_value == "PASSED"
        ):
            return values[0]

        try:
            check_captcha = client.submit(
                recaptcha_challenge_value,
                recaptcha_response_value,
                private_key=self.private_key,
                remoteip=self.get_remote_ip(),
                use_ssl=self.use_ssl,
            )

        except OSError:  # Catch timeouts, etc
            # raise IOError,"ERROR: {}".format(
            #    self.error_messages['captcha_error']
            # )
            raise ValidationError(self.error_messages["captcha_error"])

        if not check_captcha.is_valid:
            # raise IOError,"INVALID: {}".format(
            #   self.error_messages['captcha_invalid']
            # )
            raise ValidationError(self.error_messages["captcha_invalid"])

        return values[0]



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from subprocess import getstatusoutput

    pythoncmd = "python3"
except Exception:
    from commands import (  # type: ignore[import-not-found,no-redef]
        getstatusoutput,
    )

    pythoncmd = "python2"

from codenerix_lib.debugger import Debugger
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand, Debugger):
    # Show this when the user types help
    help = "Clean memcache"

    def handle(self, *args, **options):
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()

        for name, cache in settings.CACHES.items():
            if "memcache" in cache.get("BACKEND", ""):
                location = cache["LOCATION"]
                locationsp = location.split(":")
                if len(locationsp) == 1:
                    host = locationsp[0]
                    port = 11211
                else:
                    host = locationsp[0]
                    port = locationsp[1]
                self.debug(
                    "Flushing all keys for "
                    f"{name}@Memcache located at {host}:{port}",
                    color="blue",
                )
                # Get environment
                status, output = getstatusoutput(
                    f"echo 'flush_all' | nc -v -w 1 {host} {port}",
                )
                if status:
                    self.debug(
                        f"ERROR at {name}@Memcache -> {output}",
                        color="red",
                    )
                else:
                    self.debug(
                        f"OK at {name}@Memcache located "
                        f"at {host}:{port} -> {output}",
                        color="green",
                    )



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext as _

WEIGHT_UNITS = {
    # SI
    "kg": (_("Kilogram"), 1.0),  # BASE
    "g": (_("Gram"), 1000.0),
    # US/UK
    "lb": (_("Pound"), 2.2046228),
    "oz": (_("Ounce"), 35.273962),
    "st": (_("Stone"), 0.15747304),
}

LENGTH_UNITS = {
    # SI
    "m": (_("Metre"), 1.0),  # BASE
    "cm": (_("Centimetre"), 100.0),
    "km": (_("Kilometre"), 0.001),
    # US/UK
    "mi": (_("Mile"), 0.00062137119),
    "yd": (_("Yard"), 1.0936133),
    "ft": (_("Feet"), 3.2808399),
    "in": (_("Inch"), 39.3701),
}

VOLUME_UNITS = {
    # SI
    "m3": (_("Cubic metre"), 1.0),  # BASE
    "dm3": (_("Cubic decimetre"), 1000.0),
    "l": (_("Litre"), 1000.0),
    "ml": (_("Mililitre"), 1000000.0),
    # UK
    "ig": (_("UK Gallon"), 219.969),
    "ip": (_("UK Pint"), 1759.75),
    # US
    "g": (_("US Gallon"), 264.172),
    "p": (_("US Pint"), 2113.38),
    # US/UK
    "ft3": (_("Cubic feet"), 35.314666882523476),
    "yd3": (_("Cubic yard"), 1.3079506252786468),
}



#!/usr/bin/env python

import contextlib
import importlib
import logging
import types

from codenerix_lib.debugger import Debugger
from django.conf import settings
from django.core.mail import get_connection
from django.core.management.base import BaseCommand, CommandError
from django.utils.log import AdminEmailHandler

# Make sure this command is not executed inside django server
if "runserver" in settings.INSTALLED_APPS:
    raise CommandError(
        "This command cannot be executed while the Django server is running. "
        "Please stop the server and try again.",
    )

# Ensure the settings module is configured
backend_string = getattr(settings, "EMAIL_BACKEND", None)
if not backend_string:
    raise CommandError("EMAIL_BACKEND setting is not configured.")

# Attempt to import the specified email backend
try:
    backend_module, backend_class = backend_string.rsplit(".", 1)
    smtp_backend = importlib.import_module(backend_module)
except (ImportError, AttributeError) as e:
    raise CommandError(f"Invalid EMAIL_BACKEND: {e!r}")


class Command(BaseCommand, Debugger):
    # Show this when the user types help
    help = "Send a test email to admins"

    def add_arguments(self, parser):
        parser.add_argument(
            "--verbose",
            action="store_true",
            dest="verbose",
            default=False,
            help="Enable verbose output for debugging",
        )

    def handle(self, *args, **options):
        # Set verbosity level
        self.VERBOSE = options.get("verbose")

        # Autoconfigure Debugger
        self.set_name("TEST ADMIN EMAIL")
        self.set_debug()

        # Prepare handler
        self.debug("Getting AdminEmailHandler", color="blue")
        if self.VERBOSE:
            handler = self.get_patched_handler()
        else:
            handler = AdminEmailHandler()

        # Send the test email
        self.send(handler)

    def get_patched_handler(self):
        # Patch the open method of the SMTP backend to set debuglevel
        original_open = getattr(smtp_backend, backend_class).open

        # Patch the open method to enable SMTP debug
        def open_with_debug(self, *a, **kw):
            result = original_open(self, *a, **kw)
            try:
                # After each open(), force SMTP debug
                self.connection.set_debuglevel(1)
                self.stderr.write("→ [PATCH] SMTP debuglevel=1 enabled")
            except Exception:
                pass
            return result

        # Replace the open method with our patched version
        getattr(smtp_backend, backend_class).open = open_with_debug

        # Logger from smtplib to DEBUG level and output to stderr
        smtp_logger = logging.getLogger("smtplib")
        smtp_logger.setLevel(logging.DEBUG)
        smtp_ch = logging.StreamHandler(self.stderr)
        smtp_ch.setLevel(logging.DEBUG)
        smtp_logger.addHandler(smtp_ch)

        # Prepare the AdminEmailHandler to send emails to admins
        handler = AdminEmailHandler()
        handler.fail_silently = False

        # === Patch EMIT ===

        # Get original method
        orig_emit = handler.emit

        # Wrap the emit method to redirect stderr
        def emit_with_debug(record):
            with contextlib.redirect_stderr(self.stderr):
                orig_emit(record)

        # Patch the emit method to use our debug version
        handler.emit = emit_with_debug

        # === Patch SEND MAIL ===

        # Emit is calling to send_mail with a hardcoded
        # argument fail_silently=True which avoids producing
        # errors while sending emails. We want to avoid this
        # behavior because we want to DEBUG any error

        # Get the original send_mail method from the handler
        orig_send_mail = handler.send_mail

        # Wrap the send_mail method to avoid fail_silently
        def send_mail_no_fail_silently(self, *args, **kwargs):
            kwargs["fail_silently"] = False
            return orig_send_mail(*args, **kwargs)

        # Patch the send_mail method of the handler
        handler.send_mail = types.MethodType(
            send_mail_no_fail_silently,
            handler,
        )

        # === Patch connection method ===

        # Wrap the connection method to avoid fail_silently
        def connection_no_fail_silently(self, *args, **kwargs):
            return get_connection(
                backend=self.email_backend,
                fail_silently=False,
            )

        # Patch the connection method of the handler
        handler.connection = types.MethodType(
            connection_no_fail_silently,
            handler,
        )

        # Return the configured handler
        return handler

    def send(self, handler):
        # Check if the email backend is configured
        self.debug("Recording test email to admins", color="blue")
        record = logging.LogRecord(
            name="test",
            level=logging.ERROR,
            pathname=__file__,
            lineno=1,
            msg="¡Emailt TEST to admins!",
            args=(),
            exc_info=None,
        )

        # Emit the test email record
        if self.VERBOSE:
            self.debug(
                "→ Emitting test email record with AdminEmailHandler:",
                color="yellow",
            )
        else:
            self.debug("Emitting test email record", color="yellow")

        try:
            handler.emit(record)
        except Exception as e:
            self.error(f"→ An error occurred while sending the email: {e!r}")
            if self.VERBOSE:
                self.error("→ Exception details:")
                self.error(f"{e.__class__.__name__}: {e}")
                self.debug("Analysing connection data:", color="white")
                conn = get_connection(
                    backend=handler.email_backend,
                    fail_silently=False,
                )
                if conn:
                    self.debug(f"{conn.host=}", color="white")
                    self.debug(f"{conn.port=}", color="white")
                    self.debug(f"{conn.username=}", color="white")
                    self.debug(f"{conn.password=}", color="white")
                    self.debug(f"{conn.use_tls=}", color="white")
                    self.debug(f"{conn.use_ssl=}", color="white")
                    self.debug(f"{conn.timeout=}", color="white")
                    self.debug(f"{conn.ssl_keyfile=}", color="white")
                    self.debug(f"{conn.ssl_certfile=}", color="white")
                else:
                    self.error("No connection available!")
                self.error("Re-raising exception")
                raise

        if self.VERBOSE:
            self.debug(
                "→ Register sent, review above the SMTP conversion.",
                color="green",
            )
        else:
            self.debug(
                "→ Email sent to admins, check your inbox.",
                color="green",
            )
            self.debug(
                "WARNING: fail_silently is enabled by default in Django "
                "(at least until Django 5.2.2) and it may hide ERRORs. Please "
                " consider trying executing this command with --verbose for "
                "improved details, SMTP communication and debugging.",
                color="blue",
            )



from django import VERSION
from django.forms.forms import BaseForm, DeclarativeFieldsMetaclass
from django.forms.models import BaseModelForm

from .angular_base import BaseFieldsModifierMetaclass, NgFormBaseMixin
from .angular_model import NgModelFormMixin  # noqa: F401

if VERSION[:2] >= (1, 5):
    from .angular_validation import NgFormValidationMixin  # noqa: F401
if VERSION[:2] < (1, 7):
    from .models import (  # type: ignore[import-not-found] # noqa: E501
        PatchedModelFormMetaclass as ModelFormMetaclass,
    )
else:
    from django.forms.models import ModelFormMetaclass


class NgDeclarativeFieldsMetaclass(
    BaseFieldsModifierMetaclass,
    DeclarativeFieldsMetaclass,
):
    pass


class NgForm(
    NgFormBaseMixin,
    BaseForm,
    metaclass=NgDeclarativeFieldsMetaclass,
):
    """
    Convenience class to be used instead of Django's internal ``forms.Form``
    when declaring a form to be used with AngularJS.
    """


class NgModelFormMetaclass(BaseFieldsModifierMetaclass, ModelFormMetaclass):
    pass


class NgModelForm(
    NgFormBaseMixin,
    BaseModelForm,
    metaclass=NgModelFormMetaclass,
):
    """
    Convenience class to be used instead of Django's internal
    ``forms.ModelForm`` when declaring a model form to be used with AngularJS.
    """



from django.forms import widgets
from django.utils.encoding import force_str
from django.utils.html import format_html

from .angular_base import NgFormBaseMixin, SafeTuple


class NgFormValidationMixin(NgFormBaseMixin):
    """
    Add this NgFormValidationMixin to every class derived from forms.Form,
    which shall be auto validated using the Angular's validation mechanism.
    """

    add_djng_error = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for name, field in self.fields.items():
            # add ng-model to each model field
            ng_model = self.add_prefix(name)
            field.widget.attrs.setdefault("ng-model", ng_model)

    def get_field_errors(self, bound_field):
        """
        Determine the kind of input field and create a list of potential
        errors which may occur during validation of that field. This list
        is returned to be displayed in '$dirty' state if the field does
        not validate for that criteria.
        """
        errors = super().get_field_errors(
            bound_field,
        )
        if bound_field.is_hidden:
            return errors
        identifier = format_html(
            "{0}['{1}']",
            self.form_name,
            self.add_prefix(bound_field.name),
        )
        potential_errors = bound_field.field.get_potential_errors()
        errors.extend(
            [
                SafeTuple(
                    (
                        identifier,
                        self.field_error_css_classes,
                        "$dirty",
                        pe[0],
                        "invalid",
                        force_str(pe[1]),
                    ),
                )
                for pe in potential_errors
            ],
        )
        if not isinstance(bound_field.field.widget, widgets.PasswordInput):
            # all valid fields shall display OK tick after changed into
            # dirty state
            errors.append(
                SafeTuple(
                    (
                        identifier,
                        self.field_error_css_classes,
                        "$dirty",
                        "$valid",
                        "valid",
                        "",
                    ),
                ),
            )
            if bound_field.value():
                # valid bound fields shall display OK tick, even in
                # pristine state
                errors.append(
                    SafeTuple(
                        (
                            identifier,
                            self.field_error_css_classes,
                            "$pristine",
                            "$valid",
                            "valid",
                            "",
                        ),
                    ),
                )
        return errors

    def get_widget_attrs(self, bound_field):
        attrs = super().get_widget_attrs(
            bound_field,
        )
        # transfer error state from bound field to AngularJS validation
        errors = [e for e in bound_field.errors if e[3] == "$pristine"]
        if errors and self.add_djng_error:
            attrs.update({"djng-error": "bound-field"})
        # some fields require special directives to work with AngularJS
        try:
            attrs.update(
                bound_field.field.widget.get_field_attrs(bound_field.field),
            )
        except AttributeError:
            pass
        return attrs



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path

from codenerix.views import (
    LogDetails,
    LogList,
    RemoteLogCreate,
    RemoteLogDetails,
    RemoteLogList,
    status,
)

urlpatterns = [
    # Backward compatibility
    re_path(
        r"^status/(?P<status>\w+)/(?P<answer>[a-zA-Z0-9+-_/]+)$",
        status,
        name="status",
    ),
    re_path(r"^logs$", LogList.as_view(), name="codenerix_log_list"),
    re_path(
        r"^logs/(?P<pk>\w+)$",
        LogDetails.as_view(),
        name="codenerix_log_details",
    ),
    re_path(
        r"^status/(?P<status>\w+)/(?P<answer>[a-zA-Z0-9+-_/]+)$",
        status,
        name="CDNX_status",
    ),
    re_path(r"^logs$", LogList.as_view(), name="CDNX_codenerix_log_list"),
    re_path(
        r"^logs/(?P<pk>\w+)$",
        LogDetails.as_view(),
        name="CDNX_codenerix_log_details",
    ),
    re_path(
        r"^remotelogs$",
        RemoteLogList.as_view(),
        name="CDNX_remotelog_list",
    ),
    re_path(
        r"^remotelogs/add$",
        RemoteLogCreate.as_view(),
        name="CDNX_remotelog_create",
    ),
    re_path(
        r"^remotelogs/(?P<pk>\w+)$",
        RemoteLogDetails.as_view(),
        name="CDNX_remotelog_details",
    ),
]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0005_auto_20160824_1332"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_flag",
            field=models.PositiveSmallIntegerField(
                verbose_name=b"Action",  # type: ignore[arg-type]
            ),
        ),
    ]



#
# django-codenerix
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.core.exceptions import ValidationError
from django.forms import NullBooleanField
from django.forms.widgets import CheckboxInput, Select, SelectMultiple
from django.utils.translation import gettext as _

from codenerix.djng import NgForm, NgFormValidationMixin, NgModelForm
from codenerix.djng.angular_model import NgModelFormMixin
from codenerix.helpers import model_inspect
from codenerix.widgets import (
    DynamicInput,
    DynamicSelect,
    MultiDynamicSelect,
    MultiStaticSelect,
    StaticSelect,
)


class BaseForm:
    def __init__(self, *args, **kwargs):
        self.__language = None
        self.attributes = {}
        self.__codenerix_uuid = None
        self.__codenerix_request = None
        return super().__init__(*args, **kwargs)

    def set_language(self, language):
        self.__language = language

    def set_attribute(self, key, value):
        self.attributes[key] = value

    def get_name(self):
        # If name atrribute exists in Meta
        if "name" in self.Meta.__dict__:
            # Try to get name from Meta
            name = self.Meta.name
        else:
            # If not try to find it automatically
            info = model_inspect(self.Meta.model())
            if info["verbose_name"]:
                name = info["verbose_name"]
            else:
                name = info["modelname"]
        return name

    def __errors__(self):
        return []

    def clean(self):
        for field in self.fields:
            # If field not found in cleaned_data, lets try to validate the
            # value directly
            if field not in self.cleaned_data:
                # If widget is SelectMultiple remake the cleaned_data
                if isinstance(self.fields[field].widget, SelectMultiple):
                    # Validate the pks is a list of positive integers
                    pks = self.data.get(field)
                    if pks:
                        try:
                            pks = [int(pk) for pk in pks]
                        except ValueError as e:
                            msg = _("Invalid ID detected in SelectMultiple")
                            if settings.DEBUG:
                                msg += f": {e}"
                            raise ValidationError(msg)
                    else:
                        pks = []

                    # Set the cleaned_data as a queryset with the
                    # selected values
                    self.cleaned_data[field] = self.fields[
                        field
                    ].choices.queryset.filter(pk__in=pks)

                    # If field was in errors, remove it
                    if field in self._errors:
                        self._errors.pop(field)

        # Call the parent method normally
        super().clean()

    # def clean_dev(self):
    #     print("clean")
    #     # print all fields being verified
    #     print(self.errors)
    #     print(type(self.errors))

    #     for field in self.fields:
    #         print(f"field: {field}")
    #         print(f"field errors: {self.errors.get(field)}")
    #         # show the value of the field
    #         print(f"field value: {self.data.get(field)}")
    #         # show the type of the field
    #         print(f"field type: {self.fields[field].__class__}")
    #         # show the widget of the field
    #         print(f"field widget: {self.fields[field].widget.__class__}")
    #         # if field not found in cleaned_data, get the post value
    #         if field not in self.cleaned_data:
    #             # if widget is selectmultiple remake the cleaned_data
    #             if isinstance(self.fields[field].widget, SelectMultiple):
    #                 # Show the model this field is related to
    #                 for choice in self.fields[field].choices:
    #                     print(f"choice: {choice} - {choice[0]}")
    #                 print(dir(self.fields[field].choices))
    #                 self.cleaned_data[field] = self.fields[
    #                     field
    #                 ].choices.queryset.filter(pk__in=self.data.get(field))
    #                 if field in self._errors:
    #                     self._errors.pop(field)
    #         # show the type of the cleaned_data
    #         print(f"field errors: {self.errors.get(field)}")
    #         print(f"cleaned_data type: {self.cleaned_data[field].__class__}")
    #         print(f"field value: {self.cleaned_data[field]}")
    #         print("")

    #     # Call the parent method
    #     super().clean()

    #     # Get the errors
    #     errors = self.get_errors()

    #     # If there are errors
    #     if errors:
    #         # Raise an error
    #         raise OSError(errors)

    def clean_color(self):
        color = self.cleaned_data["color"]
        if len(color) != 0:
            valores_validos = "#0123456789abcdefABCDEF"
            r = True
            for lt in color:
                if lt not in valores_validos:
                    r = False
                    break
            if (
                not r
                or color[0] != "#"
                or not (len(color) == 4 or len(color) == 7)
            ):
                self._errors["color"] = [_("Invalid color")]
                return color
            else:
                return color
        else:
            return color

    def get_errors(self):
        # Where to look for fields
        if "list_errors" in dir(self):
            list_errors = self.list_errors
        else:
            # r = self.non_field_errors()
            # list_errors = [
            #   element[5]
            #   for element in list(self.non_field_errors())[:-1]
            # ]
            list_errors = []
            for element in list(self.non_field_errors())[:-1]:
                if len(element) >= 5:
                    list_errors.append(element[5])
        return list_errors

    def get_extends(self, name, userextend):
        # Set extended variables
        if userextend:
            extends = []
            for fieldextend in userextend:
                extends.append("{}__{}".format(name, fieldextend))
        else:
            extends = None
        return extends

    def __groups__(self):
        return []

    @property
    def codenerix_uuid(self):
        return self.__codenerix_uuid

    @codenerix_uuid.setter
    def codenerix_uuid(self, uuid):
        self.__codenerix_uuid = uuid
        return uuid

    @property
    def codenerix_request(self):
        return self.__codenerix_request

    @codenerix_request.setter
    def codenerix_request(self, request):
        self.__codenerix_request = request
        return request

    def get_groups(self, gs=None, processed=[], initial=True):
        """
        <--------------------------------------- 12 columns ------------------------------------>
                    <--- 6 columns --->                           <--- 6 columns --->
         ------------------------------------------   ------------------------------------------
        | Info                                     | | Personal                                 |
        |==========================================| |==========================================|
        |  -----------------   ------------------  | |                                          |
        | | Passport        | | Name             | | | Phone                          Zipcode   |
        | |=================| | [.....]  [.....] | | | [...........................]  [.......] |
        | | CID     Country | | <- 6 ->  <- 6 -> | | |       <--- 8 columns --->      <-4 col-> |
        | | [.....] [.....] | |                  | | |                                          |
        | | <- 6 -> <- 6 -> |  -----------------   | | Address                                  |
        |  -----------------                       | | [.....................................]  |
         ------------------------------------------  |           <--- 12 columns --->           |
                                                     | [..] number                              |
                                                     |           <--- 12 columns --->           |
                                                     |                                          |
                                                      ------------------------------------------
        group = [
                (_('Info'),(6,'#8a6d3b','#fcf8e3','center'),
                    (_('Identification'),6,
                        ["cid",6],
                        ["country",6],
                    ),
                    (None,6,
                        ["name",None,6],
                        ["surname",None,6,False],
                    ),
                ),
                (_('Personal'),6,
                    ["phone",None,8],
                    ["zipcode",None,4],
                    ["address",None,12],
                    ["number",None,12, True],
                ),
            ]

        Group: it is defined as tuple with 3 or more elements:
            Grammar: (<Name>, <Attributes>, <Element1>, <Element2>, ..., <ElementN>)
            If <Name> is None: no name will be given to the group and no panel decoration will be shown
            If <Size in columns> is None: default of 6 will be used

            <Attributes>:
                it can be an integer that represent the size in columns
                it can be a tuple with several attributes where each element represents:
                    (<Size in columns>,'#<Font color>','#<Background color>','<Alignment>')

            <Element>:
                it can be a Group
                it can be a Field

            Examples:
            ('Info', 6, ["name",6], ["surname",6]) -> Info panel using 6 columns with 2 boxes 6 columns for each with name and surname inputs
            ('Info', (6,None,'#fcf8e3','center'), ["name",6], ["surname",6]) -> Info panel using 6 columns with a yellow brackground in centered title, 2 boxes, 6 columns for each with name and surname inputs
            ('Info', 12, ('Name', 6, ["name",12]), ('Surname',6, ["surname",12])) -> Info panel using 12 columns with 2 panels inside
              of 6 columns each named "Name" and "Surname" and inside each of them an input "name" and "surname" where it belongs.

        Field: must be a list with at least 1 element in it:
            Grammar: [<Name of field>, <Size in columns>, <Label>]

            <Name of field>:
                This must be filled always
                It is the input's name inside the form
                Must exists as a form element or as a grouped form element

            <Size in columns>:
                Size of the input in columns
                If it is not defined or if it is defined as None: default of 6 will be used

            <Label>:
                It it is defined as False: the label for this field will not be shown
                If it is not defined or if it is defined as None: default of True will be used (default input's label will be shown)
                If it is a string: this string will be shown as a label

            Examples:
            ['age']                             Input 'age' will be shown with 6 columns and its default label
            ['age',8]                           Input 'age' will be shown with 8 columns and its default label
            ['age', None, False]                Input 'age' will be shown with 6 columns and NO LABEL
            ['age',8,False]                     Input 'age' will be shown with 8 columns and NO LABEL
            ['age',8,_("Age in days")]          Input 'age' will be shown with 8 columns and translated label text "Age in days" to user's language
            ['age',8,_("Age in days"), True]    Input 'age' will be shown with 8 columns and translated label text "Age in days" to user's language, and input inline with label
            ['age',6, None, None, None, None, None, ["ng-click=functionjs('param1')", "ng-change=functionjs2()"]]    Input 'age' with extras functions
            ['age',None,None,None,None, 'filter']    Input 'age' with extras filter ONLY DETAILS
            ['age',6, {'color': 'red'}          Input 'age' will be shown with red title

        Meta:
        =====
        - autofill: autofill system (CODENERIX)
        - extend: extended variables system (CODENERIX)
            Example: {"attr1": ["subattr1", "subattr2"]} -> it will inject in the scope attributes "subattr1" and "subattr2" from "object.attr1" if object is available (GenUpdate)
        - widgets: widgets system (DJANGO)
        """  # noqa: E501

        # Check if language is set
        if not self.__language:
            raise OSError("ERROR: No language suplied!")

        # Initialize the list
        if initial:
            processed = []

        # Where to look for fields
        if "list_fields" in dir(self):
            list_fields = self.list_fields
            check_system = "html_name"
        else:
            list_fields = self
            check_system = "name"

        # Default attributes for fields
        attributes = [
            ("columns", 6),
            ("color", None),
            ("bgcolor", None),
            ("textalign", None),
            ("inline", False),  # input in line with label
            ("label", True),
            ("extra", None),
            ("extra_div", None),
            ("foreign_info", {}),
        ]
        labels = [x[0] for x in attributes]

        # Get groups if none was given
        if gs is None:
            gs = self.__groups__()

        # Prepare the answer
        groups = []

        # Prepare focus control
        focus_first = None
        focus_must = None

        # html helper for groups and fields
        html_helper = self.html_helper()

        # Start processing
        for g in gs:
            token = {}
            token["name"] = g[0]

            if token["name"] in html_helper:
                if "pre" in html_helper[token["name"]]:
                    token["html_helper_pre"] = html_helper[token["name"]][
                        "pre"
                    ]
                if "post" in html_helper[token["name"]]:
                    token["html_helper_post"] = html_helper[token["name"]][
                        "post"
                    ]

            styles = g[1]
            if isinstance(styles, tuple):
                if len(styles) >= 1:
                    token["columns"] = g[1][0]
                if len(styles) >= 2:
                    token["color"] = g[1][1]
                if len(styles) >= 3:
                    token["bgcolor"] = g[1][2]
                if len(styles) >= 4:
                    token["textalign"] = g[1][3]
                if len(styles) >= 5:
                    token["inline"] = g[1][4]
                if len(styles) >= 7:
                    token["extra"] = g[1][5]
                if len(styles) >= 8:
                    token["extra_div"] = g[1][6]
            else:
                token["columns"] = g[1]
            fs = g[2:]
            fields = []
            for f in fs:
                # Field
                atr = {}
                # Decide weather this is a Group or not
                if isinstance(f, tuple):
                    # Recursive
                    fields += self.get_groups([list(f)], processed, False)
                else:
                    try:
                        list_type = [
                            str,
                            unicode,
                        ]
                    except NameError:
                        list_type = [
                            str,
                        ]
                    # Check if it is a list
                    if isinstance(f, list):
                        # This is a field with attributes, get the name
                        field = f[0]

                        if (
                            html_helper
                            and token["name"] in html_helper
                            and "items" in html_helper[token["name"]]
                            and field in html_helper[token["name"]]["items"]
                        ):
                            if (
                                "pre"
                                in html_helper[token["name"]]["items"][field]
                            ):
                                atr["html_helper_pre"] = html_helper[
                                    token["name"]
                                ]["items"][field]["pre"]
                            if (
                                "post"
                                in html_helper[token["name"]]["items"][field]
                            ):
                                atr["html_helper_post"] = html_helper[
                                    token["name"]
                                ]["items"][field]["post"]

                        # Process each attribute (if any)
                        dictionary = False
                        for idx, element in enumerate(f[1:]):
                            if isinstance(element, dict):
                                dictionary = True
                                for key in element.keys():
                                    if key in labels:
                                        atr[key] = element[key]
                                    else:
                                        raise OSError(
                                            f"Unknown attribute '{key}' as "
                                            f"field '{field}' in list "
                                            "of fields",
                                        )
                            else:
                                if not dictionary:
                                    if element is not None:
                                        atr[attributes[idx][0]] = element
                                else:
                                    raise OSError(
                                        "We already processed a dicionary "
                                        "element in this list of fields, you "
                                        "can not add anoother type of "
                                        "elements to it, you must keep going "
                                        "with dictionaries",
                                    )
                    elif type(f) in list_type:
                        field = f
                    else:
                        raise OSError(
                            f"Uknown element type '{type(f)}' inside "
                            f"group '{token['name']}'",
                        )

                    # Get the Django Field object
                    found = None
                    foundbool = False
                    userwidget = None
                    userextend = None
                    for infield in list_fields:
                        if infield.__dict__[check_system] == field:
                            found = infield
                            foundbool = True

                            # Check if the user specified a extend
                            if "widgets" in dir(self.Meta):
                                userwidget = self.Meta.widgets.get(field, None)
                            # Check if the user specified a extend
                            if "extend" in dir(self.Meta):
                                userextend = self.Meta.extend.get(field, None)
                            break

                    if foundbool:
                        # Get attributes (required and original attributes)
                        wrequired = found.field.widget.is_required
                        wattrs = found.field.widget.attrs

                        # Fill base attributes
                        atr["name"] = found.html_name
                        atr["input"] = found
                        atr["inputbool"] = foundbool
                        atr["focus"] = False
                        atr["extend"] = self.get_extends(
                            found.html_name,
                            userextend,
                        )

                        # Set focus
                        if focus_must is None:
                            if focus_first is None:
                                focus_first = atr
                            if wrequired:
                                focus_must = atr

                        # Autocomplete
                        if "autofill" in dir(self.Meta):
                            autofill = self.Meta.autofill.get(
                                found.html_name,
                                None,
                            )
                            atr["autofill"] = autofill

                            if autofill:
                                # Check format of the request
                                autokind = autofill[0]
                                if isinstance(autokind, str):
                                    # Replace widget if the user didn't
                                    # define any
                                    if not userwidget:
                                        # Using new format
                                        if autokind == "select":
                                            # If autofill is True for this
                                            # field set the DynamicSelect
                                            # widget
                                            found.field.widget = DynamicSelect(
                                                wattrs,
                                            )

                                        elif autokind == "multiselect":
                                            # If autofill is True for this
                                            # field set the DynamicSelect
                                            # widget
                                            found.field.widget = (
                                                MultiDynamicSelect(wattrs)
                                            )

                                        elif autokind == "input":
                                            # If autofill is True for this
                                            # field set the DynamicSelect
                                            # widget
                                            found.field.widget = DynamicInput(
                                                wattrs,
                                            )

                                        else:
                                            raise OSError(
                                                "Autofill filled using new "
                                                "format but autokind is "
                                                f"'{autokind}' and I only "
                                                "know 'input' or 'select'",
                                            )

                                    # Configure widget
                                    found.field.widget.is_required = wrequired
                                    found.field.widget.form_name = (
                                        self.form_name
                                    )
                                    found.field.widget.field_name = (
                                        infield.html_name
                                    )
                                    found.field.widget.autofill_deepness = (
                                        autofill[1]
                                    )
                                    found.field.widget.autofill_url = autofill[
                                        2
                                    ]
                                    found.field.widget.autofill = autofill[3:]

                                else:
                                    # Get old information [COMPATIBILITY WITH
                                    # OLD VERSION]

                                    # Replace widget if the user didn't
                                    # define any
                                    if not userwidget:
                                        # If autofill is True for this field
                                        # set the DynamicSelect widget
                                        found.field.widget = DynamicSelect(
                                            wattrs,
                                        )

                                    # Configure widget
                                    found.field.widget.is_required = wrequired
                                    found.field.widget.form_name = (
                                        self.form_name
                                    )
                                    found.field.widget.field_name = (
                                        infield.html_name
                                    )
                                    found.field.widget.autofill_deepness = (
                                        autofill[0]
                                    )
                                    found.field.widget.autofill_url = autofill[
                                        1
                                    ]
                                    found.field.widget.autofill = autofill[2:]
                        else:
                            # Set we don't have autofill for this field
                            atr["autofill"] = None

                        # Check if we have to replace the widget with a
                        # newer one
                        if isinstance(
                            found.field.widget,
                            Select,
                        ) and not isinstance(
                            found.field.widget,
                            DynamicSelect,
                        ):
                            # Replace widget if the user didn't define any and
                            # we haven't done yet
                            if (
                                (not userwidget)
                                and (
                                    not isinstance(
                                        found.field.widget,
                                        MultiStaticSelect,
                                    )
                                )
                                and (
                                    not isinstance(
                                        found.field.widget,
                                        MultiDynamicSelect,
                                    )
                                )
                            ):
                                if isinstance(
                                    found.field.widget,
                                    SelectMultiple,
                                ):
                                    found.field.widget = MultiStaticSelect(
                                        wattrs,
                                    )
                                else:
                                    found.field.widget = StaticSelect(wattrs)
                            if hasattr(
                                found.field.widget,
                                "choices",
                            ) and hasattr(found.field, "choices"):
                                found.field.widget.choices = (
                                    found.field.choices
                                )
                            found.field.widget.is_required = wrequired
                            found.field.widget.form_name = self.form_name
                            found.field.widget.field_name = infield.html_name

                        # Fill all attributes
                        for attribute, default in attributes:
                            if attribute not in atr.keys():
                                atr[attribute] = default

                        # Check if we have to remove foreignkey buttons
                        if isinstance(
                            found.field.widget,
                            SelectMultiple,
                        ):
                            atr["foreign_info"] = None

                        # Fill label
                        if atr["label"] is True:
                            atr["label"] = found.label
                        # Set language
                        flang = getattr(found.field, "set_language", None)
                        if flang:
                            flang(self.__language)
                        flang = getattr(
                            found.field.widget,
                            "set_language",
                            None,
                        )
                        if flang:
                            flang(self.__language)
                        # Attach the element
                        fields.append(atr)
                        # Remember we have processed it
                        processed.append(found.__dict__[check_system])
                    else:
                        raise OSError(
                            f"Unknown field '{f}' specified in "
                            f"group '{token['name']}'",
                        )

            token["fields"] = fields
            groups.append(token)

        # Add the rest of attributes we didn't use yet
        if initial:
            fields = []
            for infield in list_fields:
                if infield.__dict__[check_system] not in processed:
                    # Check if the user specified a widget
                    if "widgets" in dir(self.Meta):
                        userwidget = self.Meta.widgets.get(
                            infield.html_name,
                            None,
                        )
                    else:
                        userwidget = None

                    # Check if the user specified a extend
                    if "extend" in dir(self.Meta):
                        userextend = self.Meta.extend.get(
                            infield.html_name,
                            None,
                        )
                    else:
                        userextend = None

                    # Get attributes (required and original attributes)
                    wattrs = infield.field.widget.attrs
                    wrequired = infield.field.widget.is_required

                    # Prepare attr
                    atr = {}

                    # Fill base attributes
                    atr["name"] = infield.html_name
                    atr["input"] = infield
                    atr["inputbool"] = True
                    atr["focus"] = False
                    atr["extend"] = self.get_extends(
                        infield.html_name,
                        userextend,
                    )

                    # Set focus
                    if focus_must is None:
                        if focus_first is None:
                            focus_first = atr
                        if wrequired:
                            focus_must = atr

                    # Autocomplete
                    if "autofill" in dir(self.Meta):
                        autofill = self.Meta.autofill.get(
                            infield.html_name,
                            None,
                        )
                        atr["autofill"] = autofill

                        if autofill:
                            # Check format of the request
                            autokind = autofill[0]

                            if isinstance(autokind, str):
                                # Get old information

                                # Replace widget if the user didn't define any
                                if not userwidget:
                                    # Using new format
                                    if autokind == "select":
                                        # If autofill is True for this field
                                        # set the DynamicSelect widget
                                        infield.field.widget = DynamicSelect(
                                            wattrs,
                                        )
                                    elif autokind == "multiselect":
                                        # If autofill is True for this field
                                        # set the DynamicSelect widget
                                        infield.field.widget = (
                                            MultiDynamicSelect(wattrs)
                                        )
                                    elif autokind == "input":
                                        # If autofill is True for this field
                                        # set the DynamicSelect widget
                                        infield.field.widget = DynamicInput(
                                            wattrs,
                                        )
                                    else:
                                        raise OSError(
                                            "Autofill filled using new "
                                            "format but autokind is "
                                            f"'{autokind}' and I only know "
                                            "'input' or 'select'",
                                        )

                                # Configure widget
                                infield.field.widget.is_required = wrequired
                                infield.field.widget.form_name = self.form_name
                                infield.field.widget.field_name = (
                                    infield.html_name
                                )
                                infield.field.widget.autofill_deepness = (
                                    autofill[1]
                                )
                                infield.field.widget.autofill_url = autofill[2]
                                infield.field.widget.autofill = autofill[3:]
                            else:
                                # Get old information [COMPATIBILITY WITH
                                # OLD VERSION]

                                # Replace widget if the user didn't define any
                                if not userwidget:
                                    # If autofill is True for this field set
                                    # the DynamicSelect widget
                                    infield.field.widget = DynamicSelect(
                                        wattrs,
                                    )

                                # Configure widget
                                infield.field.widget.is_required = wrequired
                                infield.field.widget.form_name = self.form_name
                                infield.field.widget.field_name = (
                                    infield.html_name
                                )
                                infield.field.widget.autofill_deepness = (
                                    autofill[0]
                                )
                                infield.field.widget.autofill_url = autofill[1]
                                infield.field.widget.autofill = autofill[2:]
                    else:
                        # Set we don't have autofill for this field
                        atr["autofill"] = None

                    # Check if we have to replace the widget with a newer one
                    if isinstance(
                        infield.field.widget,
                        Select,
                    ) and not isinstance(infield.field.widget, DynamicSelect):
                        # Replace widget if the user didn't define any
                        if not userwidget:
                            if isinstance(infield.field, NullBooleanField):
                                infield.field.widget = CheckboxInput(wattrs)
                            elif not isinstance(
                                infield.field.widget,
                                MultiStaticSelect,
                            ) and not isinstance(
                                infield.field.widget,
                                MultiDynamicSelect,
                            ):
                                if isinstance(
                                    infield.field.widget,
                                    SelectMultiple,
                                ):
                                    infield.field.widget = MultiStaticSelect(
                                        wattrs,
                                    )
                                else:
                                    infield.field.widget = StaticSelect(wattrs)

                        # Configure widget
                        if hasattr(
                            infield.field.widget,
                            "choices",
                        ) and hasattr(infield.field, "choices"):
                            infield.field.widget.choices = (
                                infield.field.choices
                            )
                        infield.field.widget.is_required = wrequired
                        infield.field.widget.form_name = self.form_name
                        infield.field.widget.field_name = infield.html_name

                    # Fill all attributes
                    for attribute, default in attributes:
                        if attribute not in atr.keys():
                            atr[attribute] = default

                    # Check if we have to remove foreignkey buttons
                    if isinstance(
                        infield.field.widget,
                        SelectMultiple,
                    ):
                        atr["foreign_info"] = None

                    # Fill label
                    if atr["label"] is True:
                        atr["label"] = infield.label
                    # Set language
                    flang = getattr(infield.field, "set_language", None)
                    if flang:
                        flang(self.__language)
                    flang = getattr(infield.field.widget, "set_language", None)
                    if flang:
                        flang(self.__language)
                    # Attach the attribute
                    fields.append(atr)

            # Save the new elements
            if fields:
                groups.append({"name": None, "columns": 12, "fields": fields})

        # Set focus
        if focus_must:
            focus_must["focus"] = True
        elif focus_first is not None:
            focus_first["focus"] = True

        # Return the resulting groups
        return groups

    def html_helper(self):
        """
        g={'Group Name':{
            'pre': 'text pre div',
            'post': 'text post div',
            'items': {
                'input name':{'pre': 'text pre name', 'post': 'text post name'},
                'example':{'pre': '<p>text <b>for</b> help</p>', 'post': '<div>more help</div>'},
                }
            }
        }
        """  # noqa: E501
        return {}


class GenModelForm(
    BaseForm,
    NgModelFormMixin,
    NgFormValidationMixin,
    NgModelForm,
):
    def save(self, *args, **kwargs):
        # Prepare the instance
        self.instance.codenerix_uuid = self.codenerix_uuid
        self.instance.codenerix_request = self.codenerix_request
        # Save the object
        obj = super().save(*args, **kwargs)
        # Return the object
        return obj

    pass


class GenForm(BaseForm, NgFormValidationMixin, NgForm):
    add_djng_error = False

    class Meta:
        name = ""



# Generated by Django 1.10.7 on 2017-09-25 05:39

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("codenerix", "0019_auto_20170725_1822"),
    ]

    operations = [
        migrations.CreateModel(
            name="RemoteLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True,
                        verbose_name="Created",
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(
                        auto_now=True,
                        verbose_name="Updated",
                    ),
                ),
                ("data", models.TextField(verbose_name="Data")),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "list",
                ),
            },
        ),
    ]
